% The dynamic model of n-connected bogies. This function calls
% 'bogie_friction_Karnopp.m' for calculating friction forces using karnopp
% model.
% Author: Babak Sedghi, LA-EPFL, babak.sedghi@epfl.ch
% Revision: 1, Date: 15. 5.2004

function [sys,x0,str,ts]=GenerationTrajectoire(t,x,u,flag,Param)

switch flag

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  case 0         
    [sys,x0,str,ts] = mdlInitializeSizes(Param);

  %%%%%%%%%%%%%%%
  % Derivatives %
  %%%%%%%%%%%%%%%
  case 1 
      
    sys = mdlDerivatives(t,x,u,Param);

  %%%%%%%%%%%%%%%%%%%%%%%%
  % Update and Terminate %
  %%%%%%%%%%%%%%%%%%%%%%%%
  case {2,9}
    sys = []; % do nothing

  %%%%%%%%%%
  % Output %
  %%%%%%%%%%
  case 3
      %keyboard
    sys = mdlOutputs(t,x,u,Param); 
    
  otherwise
    error(['unhandled flag = ',num2str(flag)]);
end

% end limintm

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes(Param)

sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 20;
sizes.NumInputs      = 0;
sizes.DirFeedthrough = 1;
sizes.NumSampleTimes = 1;

sys = simsizes(sizes);
str = [];

x0  = [];
ts  = [0 0];   % sample time: [period, offset]

% end mdlInitializeSizes

%

%=============================================================================
% mdlDerivatives
% Compute derivatives for continuous states.
%=============================================================================
%
function sys = mdlDerivatives(t,x,u,Param)
sys=[];
% end mdlDerivatives

%
%=============================================================================
% mdlOutputs
% Return the output vector for the S-function
%=============================================================================
%
function sys = mdlOutputs(t,x,u,Param)

% fre1 = u(1);
% fre2 = u(2);
% rayon = u(3);
% initial_x = u(4);
% final_x = u(5);
% initial_time = u(6);
% final_time = u(7);
% ChoixTrajectoire = u(8);

fre1 = evalin('base','fre1');
fre2 = evalin('base','fre2');
rayon = evalin('base','rayon');
initial_x = evalin('base','initial_x');
final_x = evalin('base','final_x');
initial_time = evalin('base','initial_time');
final_time = evalin('base','final_time');
ChoixTrajectoire = evalin('base','ChoixTrajectoire');

switch ChoixTrajectoire

  case 0 
      
     x1     = final_x(1);
     x1d    = 0;
     x1dd   = 0;
     x1ddd  = 0;
     x1dddd = 0;
      
     x2     = final_x(2);
     x2d    = 0;
     x2dd   = 0;
     x2ddd  = 0;
     x2dddd = 0;
     
     x3     = final_x(3);
     x3d    = 0;
     x3dd   = 0;
     x3ddd  = 0;
     x3dddd = 0;
     
     x03     = final_x(4);
     x03d    = 0;
     x03dd   = 0;
     x03ddd  = 0;
     x03dddd = 0;
     
  case 1 
   
     [x1,x1d,x1dd,x1ddd,x1dddd,x2,x2d,x2dd,x2ddd,x2dddd,x3,x3d,x3dd,x3ddd,x3dddd] = TrajPoly(initial_x, final_x,final_time-initial_time,t-initial_time);
     
     x03     = final_x(4);
     x03d    = 0;
     x03dd   = 0;
     x03ddd  = 0;
     x03dddd = 0;
     
  case 2 
      
     [x1,x1d,x1dd,x1ddd,x1dddd,x2,x2d,x2dd,x2ddd,x2dddd,x3,x3d,x3dd,x3ddd,x3dddd] = TrajCercle(initial_x,final_x,rayon,fre1,final_time-initial_time,t-initial_time);
     
     coor_x = initial_x(1)-rayon;
     coor_y = initial_x(2);
     x1     = x1 + coor_x;
     x2     = x2 + coor_y;
     
     x03     = final_x(4);
     x03d    = 0;
     x03dd   = 0;
     x03ddd  = 0;
     x03dddd = 0;
  
   case 3
     [x1,x1d,x1dd,x1ddd,x1dddd,x2,x2d,x2dd,x2ddd,x2dddd,x3,x3d,x3dd,x3ddd,x3dddd] = Lissajou(initial_x,final_x,rayon,fre1,fre2,final_time-initial_time,t-initial_time);
     
     coor_x = initial_x(1)-rayon;
     coor_y = initial_x(2);
     x1     = x1 + coor_x;
     x2     = x2 + coor_y;
     
     x03     = final_x(4);
     x03d    = 0;
     x03dd   = 0;
     x03ddd  = 0;
     x03dddd = 0;
  
    otherwise
    error(['unhandled flag = ',num2str(flag)]);
end


sys(1)  = x1;
sys(2)  = x1d;
sys(3)  = x1dd;
sys(4)  = x1ddd;
sys(5)  = x1dddd;

sys(6)  = x2;
sys(7)  = x2d;
sys(8)  = x2dd;
sys(9)  = x2ddd;
sys(10) = x2dddd;

sys(11)  = x3;
sys(12)  = x3d;
sys(13)  = x3dd;
sys(14)  = x3ddd;
sys(15)  = x3dddd;

sys(16)  = x03;
sys(17)  = x03d;
sys(18)  = x03dd;
sys(19)  = x03ddd;
sys(20)  = x03dddd;

% end mdlOutputs


